@@ -6,7 +6,7 @@
 #
 ###
 #
-#  Copyright (c) 1995 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
+#  Copyright (c) 1995 - 97	David Albert Bagley, bagleyd@bigfoot.com
 #
 #                   All Rights Reserved
 #
@@ -31,14 +31,20 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
 #ifdef VMS
 #include <unixlib.h>
-#define getlogin cuserid
+#define getlogin() cuserid(NULL)
 #else
-#ifndef apollo
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #endif
+#if HAVE_FCNTL_H
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#endif
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
@@ -69,25 +75,37 @@
 #define MAXFACES 6
 
 #define MAXRECORD 32767
+#define FILENAMELEN 1024
+#define USERNAMELEN 128
+#define NOACCESS "noaccess"
+#define NOBODY "nobody"
+
+typedef struct {
+	int         score;
+	char        name[USERNAMELEN];
+} GameRecord;
 
 static void Initialize(void);
 static void CallbackDino(Widget w, caddr_t clientData, dinoCallbackStruct * callData);
 
 static void PrintRecord(int mode, Boolean orient, Boolean practice);
 static int  HandleSolved(int counter, int mode, Boolean orient);
+static void InitRecords(void);
 static void ReadRecords(void);
 static void WriteRecords(void);
 
 static void ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct * cbs);
 static void OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs);
 static void PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs);
-static void motif_print(Widget w, char * text);
+static void motif_print(Widget w, char *text);
 
 static Arg  arg[5];
 static Widget moves, record, message, dino2d, dino3d, modes[MAXMODES], orientSwitch,
             practiceSwitch;
-static int  dinoRecord[MAXMODES][2], movesDsp = 0;
+static GameRecord dinoRecord[MAXMODES][2];
+static int  movesDsp = 0;
 static char messageDsp[128] = "Welcome";
+static char usernameDsp[USERNAMELEN] = "";
 static char buff[256];
 
 static char *modeString[] =
@@ -102,35 +120,40 @@ Usage(void)
 	(void) fprintf(stderr,
 	     "\t[-geometry [{width}][x{height}][{+-}{xoff}[{+-}{yoff}]]]\n");
 	(void) fprintf(stderr,
-		"\t[-display [{host}]:[{vs}]][-fg {color}] [-bg {color}]\n");
+		       "\t[-display [{host}]:[{vs}]] [-mono]\n");
 	(void) fprintf(stderr,
-		       "\t[-{border|bd} {color}] [-mono] [-{mode {int} | both}] [-[no]orient]\n");
+		"\t[-{foreground|fg} {color}] [-{background|bg} {color}]\n");
 	(void) fprintf(stderr,
-		       "\t-[no]practice] [-face{0|1|2|3|4|5} {color}]\n");
+		  "\t[-{border|bd} {color}] [-face{0|1|2|3|4|5} {color}]\n");
+	(void) fprintf(stderr,
+		 "\t[-{mode {int} | both}] [-[no]orient] [-[no]practice]\n");
+	(void) fprintf(stderr,
+		       "\t[-username {string}]\n");
 	exit(1);
 }
 
 static XrmOptionDescRec options[] =
 {
-	{"-fg", "*dino.Foreground", XrmoptionSepArg, NULL},
-	{"-bg", "*Background", XrmoptionSepArg, NULL},
-	{"-foreground", "*dino.Foreground", XrmoptionSepArg, NULL},
-	{"-background", "*Background", XrmoptionSepArg, NULL},
-	{"-border", "*dino.pieceBorder", XrmoptionSepArg, NULL},
-	{"-bd", "*dino.pieceBorder", XrmoptionSepArg, NULL},
 	{"-mono", "*dino.mono", XrmoptionNoArg, "TRUE"},
-	{"-mode", "*dino.mode", XrmoptionSepArg, NULL},
-	{"-both", "*dino.mode", XrmoptionNoArg, "4"},
-	{"-orient", "*dino.orient", XrmoptionNoArg, "TRUE"},
-	{"-noorient", "*dino.orient", XrmoptionNoArg, "FALSE"},
-	{"-practice", "*dino.practice", XrmoptionNoArg, "TRUE"},
-	{"-nopractice", "*dino.practice", XrmoptionNoArg, "FALSE"},
+	{"-fg", "*dino.Foreground", XrmoptionSepArg, NULL},
+	{"-foreground", "*dino.Foreground", XrmoptionSepArg, NULL},
+	{"-bg", "*Background", XrmoptionSepArg, NULL},
+	{"-background", "*Background", XrmoptionSepArg, NULL},
+	{"-bd", "*dino.pieceBorder", XrmoptionSepArg, NULL},
+	{"-border", "*dino.pieceBorder", XrmoptionSepArg, NULL},
 	{"-face0", "*dino.faceColor0", XrmoptionSepArg, NULL},
 	{"-face1", "*dino.faceColor1", XrmoptionSepArg, NULL},
 	{"-face2", "*dino.faceColor2", XrmoptionSepArg, NULL},
 	{"-face3", "*dino.faceColor3", XrmoptionSepArg, NULL},
 	{"-face4", "*dino.faceColor4", XrmoptionSepArg, NULL},
 	{"-face5", "*dino.faceColor5", XrmoptionSepArg, NULL},
+	{"-mode", "*dino.mode", XrmoptionSepArg, NULL},
+	{"-both", "*dino.mode", XrmoptionNoArg, "4"},
+	{"-orient", "*dino.orient", XrmoptionNoArg, "TRUE"},
+	{"-noorient", "*dino.orient", XrmoptionNoArg, "FALSE"},
+	{"-practice", "*dino.practice", XrmoptionNoArg, "TRUE"},
+	{"-nopractice", "*dino.practice", XrmoptionNoArg, "FALSE"},
+	{"-username", "*dino.userName", XrmoptionSepArg, NULL}
 };
 
 int
@@ -266,13 +289,14 @@ main(int argc, char **argv)
 /* There's probably a better way to assure that they are the same but I don't
    know it off hand. */
 static void
-MakeEquivalent(int *mode, Boolean * orient, Boolean * practice)
+MakeEquivalent(String * username, int *mode, Boolean * orient, Boolean * practice)
 {
 	Boolean     mono;
 	Pixel       foreground, background, pieceBorder;
 	String      faceColor[MAXFACES];
 
 	XtVaGetValues(dino2d,
+		      XtNuserName, username,
 		      XtNmode, mode,
 		      XtNorient, orient,
 		      XtNpractice, practice,
@@ -292,6 +316,7 @@ MakeEquivalent(int *mode, Boolean * orient, Boolean * practice)
 		      XtNstart, FALSE,
 		      NULL);
 	XtVaSetValues(dino3d,
+		      XtNuserName, *username,
 		      XtNmode, *mode,
 		      XtNorient, *orient,
 		      XtNpractice, *practice,
@@ -315,12 +340,22 @@ Initialize(void)
 {
 	int         mode;
 	Boolean     orient, practice;
+	String      username;
 
-	MakeEquivalent(&mode, &orient, &practice);
-	ReadRecords();
+	MakeEquivalent(&username, &mode, &orient, &practice);
 	XmToggleButtonSetState(modes[mode - PERIOD2], True, False);
 	XmToggleButtonSetState(orientSwitch, orient, True);
 	XmToggleButtonSetState(practiceSwitch, practice, True);
+	InitRecords();
+	ReadRecords();
+	(void) strcpy(usernameDsp, username);
+	if (!strcmp(usernameDsp, "") || !strcmp(usernameDsp, NOACCESS) ||
+	    !strcmp(usernameDsp, NOBODY)) {
+		/* The NOACCESS is not necasary, but it stops people from being cute. */
+		(void) sprintf(usernameDsp, "%s", getlogin());
+		if (!strcmp(usernameDsp, "") || !strcmp(usernameDsp, NOACCESS))
+			(void) sprintf(usernameDsp, "%s", NOBODY);	/* It really IS nobody */
+	}
 	PrintRecord(mode, orient, practice);
 }
 
@@ -380,7 +415,7 @@ CallbackDino(Widget w, caddr_t clientData, dinoCallbackStruct * callData)
 				movesDsp = 0;
 			else {
 				if (HandleSolved(movesDsp, mode, orient))
-					(void) sprintf(messageDsp, "Congratulations %s!!", getlogin());
+					(void) sprintf(messageDsp, "Congratulations %s!!", usernameDsp);
 				else
 					(void) strcpy(messageDsp, "Solved!");
 			}
@@ -444,8 +479,8 @@ CallbackDino(Widget w, caddr_t clientData, dinoCallbackStruct * callData)
 			break;
 	}
 	motif_print(message, messageDsp);
-  (void) sprintf(buff, "%d", movesDsp);
-  motif_print(moves, buff);
+	(void) sprintf(buff, "%d", movesDsp);
+	motif_print(moves, buff);
 }
 
 static void
@@ -512,8 +547,8 @@ PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cb
 		      XtNpractice, &orient,
 		      NULL);
 	movesDsp = 0;
-  (void) sprintf(buff, "%d", movesDsp);
-  motif_print(moves, buff);
+	(void) sprintf(buff, "%d", movesDsp);
+	motif_print(moves, buff);
 	if (!practice)
 		(void) strcpy(messageDsp, "Randomize to start");
 	PrintRecord(mode, orient, practice);
@@ -527,12 +562,14 @@ PrintRecord(int mode, Boolean orient, Boolean practice)
 
 	if (practice)
 		motif_print(record, "practice");
-	else if (dinoRecord[i][j] >= MAXRECORD)
-		motif_print(record, "NEVER");
-	else {
-    (void) sprintf(buff, "%d", dinoRecord[i][j]);
-    motif_print(record, buff);
-  }
+	else if (dinoRecord[i][j].score >= MAXRECORD) {
+		(void) sprintf(buff, "NEVER %s", NOACCESS);
+		motif_print(record, buff);
+	} else {
+		(void) sprintf(buff, "%d %s",
+			       dinoRecord[i][j].score, dinoRecord[i][j].name);
+		motif_print(record, buff);
+	}
 }
 
 static int
@@ -541,10 +578,13 @@ HandleSolved(int counter, int mode, Boolean orient)
 	int         i = mode - PERIOD2;
 	int         j = (orient) ? 1 : 0;
 
-	if (counter < dinoRecord[i][j]) {
-		dinoRecord[i][j] = counter;
-		if (orient && (dinoRecord[i][j] < dinoRecord[!i][j]))
-			dinoRecord[!i][j] = counter;
+	if (counter < dinoRecord[i][j].score) {
+		dinoRecord[i][j].score = counter;
+		(void) strcpy(dinoRecord[i][j].name, usernameDsp);
+		if (orient && (counter < dinoRecord[!i][j].score)) {
+			dinoRecord[!i][j].score = counter;
+			(void) strcpy(dinoRecord[!i][j].name, usernameDsp);
+		}
 		WriteRecords();
 		PrintRecord(mode, orient, FALSE);
 		return TRUE;
@@ -552,23 +592,36 @@ HandleSolved(int counter, int mode, Boolean orient)
 	return FALSE;
 }
 
+static void
+InitRecords(void)
+{
+	int         mode, orient;
+
+	for (mode = 0; mode < MAXMODES; mode++)
+		for (orient = 0; orient < 2; orient++) {
+			dinoRecord[mode][orient].score = MAXRECORD;
+			(void) strcpy(dinoRecord[mode][orient].name, NOACCESS);
+		}
+}
+
 static void
 ReadRecords(void)
 {
 	FILE       *fp;
 	int         n, mode, orient;
+	char        username[USERNAMELEN];
 
-	for (mode = 0; mode < MAXMODES; mode++)
-		for (orient = 0; orient < 2; orient++)
-			dinoRecord[mode][orient] = MAXRECORD;
 	if ((fp = fopen(SCOREFILE, "r")) == NULL) {
-    (void) sprintf(buff, "Can not open %s, taking defaults.", SCOREFILE);
-    motif_print(message, buff);
-  } else {
+		(void) sprintf(buff, "Can not open %s, taking defaults.", SCOREFILE);
+		motif_print(message, buff);
+	} else {
 		for (mode = 0; mode < MAXMODES; mode++)
 			for (orient = 0; orient < 2; orient++) {
-				(void) fscanf(fp, "%d", &n);
-				dinoRecord[mode][orient] = n;
+				(void) fscanf(fp, "%d %s\n", &n, username);
+				if (n <= dinoRecord[mode][orient].score) {
+					dinoRecord[mode][orient].score = n;
+					(void) strcpy(dinoRecord[mode][orient].name, username);
+				}
 			}
 		(void) fclose(fp);
 	}
@@ -581,14 +634,38 @@ WriteRecords(void)
 	int         mode, orient;
 
 	if ((fp = fopen(SCOREFILE, "w")) == NULL) {
-    (void) sprintf(buff, "Can not write to %s.", SCOREFILE);
-    motif_print(message, buff);
-  } else {
+		(void) sprintf(buff, "Can not write to %s.", SCOREFILE);
+		motif_print(message, buff);
+	} else {
+#if HAVE_FCNTL_H
+		int         lfd;
+		char        lockfile[FILENAMELEN];
+
+		(void) strcpy(lockfile, SCOREFILE);
+		(void) strcat(lockfile, ".lock");
+		while (((lfd = open(lockfile, O_CREAT | O_EXCL, 0644)) < 0) &&
+		       errno == EEXIST)
+			(void) sleep(1);
+		if (lfd < 0) {
+#if 1
+			(void) fprintf(stderr, "Lock file exists... guessing its an old one.\n");
+#else
+			(void) fprintf(stderr, "Lock file exists... score not recorded - sorry.\n"
+				);
+			return;
+#endif
+		}
+#endif
 		for (mode = 0; mode < MAXMODES; mode++) {
 			for (orient = 0; orient < 2; orient++)
-				(void) fprintf(fp, "%d ", dinoRecord[mode][orient]);
+				(void) fprintf(fp, "%d %s\n",
+					       dinoRecord[mode][orient].score, dinoRecord[mode][orient].name);
 			(void) fprintf(fp, "\n");
 		}
+#if HAVE_FCNTL_H
+		(void) close(lfd);
+		(void) unlink(lockfile);
+#endif
 		(void) fclose(fp);
 	}
 }
