@@ -52,6 +52,10 @@
 #define XtNwedgeColor5 "wedgeColor5"
 #define XtNwedgeColor6 "wedgeColor6"
 #define XtNwedgeColor7 "wedgeColor7"
+#define XtNwedgeColor8 "wedgeColor8"
+#define XtNwedgeColor9 "wedgeColor9"
+#define XtNwedgeColor10 "wedgeColor10"
+#define XtNwedgeColor11 "wedgeColor11"
 #define XtCUserName "UserName"
 #define XtCMono "Mono"
 #define XtCWedges "Wedges"
@@ -74,8 +78,10 @@
 #define MBALL_WEDGE4 9
 #define MBALL_WEDGE6 10
 #define MBALL_WEDGE8 11
-#define MBALL_COMPUTED 12
-#define MBALL_UNDO 13
+#define MBALL_WEDGE10 12
+#define MBALL_WEDGE12 13
+#define MBALL_COMPUTED 14
+#define MBALL_UNDO 15
 
 typedef struct _MballClassRec *MballWidgetClass;
 typedef struct _MballRec *MballWidget;
