@@ -67,7 +67,7 @@
 
 /* The following are in MballP.h also */
 #define MINWEDGES 2
-#define MAXWEDGES 8
+#define MAXWEDGES 12
 #define MINRINGS 1
 
 #define MAXRINGS 6
@@ -109,7 +109,7 @@ static char buff[256];
 
 static char *wedgeString[] =
 {
-	"Two", "Four", "Six", "Eight"
+	"Two", "Four", "Six", "Eight", "Ten", "Twelve"
 };
 
 static void
@@ -422,6 +422,8 @@ CallbackMball(Widget w, caddr_t clientData, mballCallbackStruct * callData)
 		case MBALL_WEDGE4:
 		case MBALL_WEDGE6:
 		case MBALL_WEDGE8:
+		case MBALL_WEDGE10:
+		case MBALL_WEDGE12:
 			movesDsp = 0;
 			wedges = 2 * (callData->reason - MBALL_WEDGE2) + MINWEDGES;
 			PrintRecord(wedges, rings, orient, practice);
