@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1996	David Albert Bagley, bagleyd@hertz.njit.edu
+#  Copyright (c) 1996 - 97	David Albert Bagley, bagleyd@bigfoot.com
 #
 #                   All Rights Reserved
 #
@@ -30,7 +30,7 @@
 #ifdef VMS
 #include <unixlib.h>
 #else
-#ifndef apollo
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #endif
@@ -73,7 +73,7 @@ static void DrawPyramid(PanexWidget w, int color, int i, int j, int size, int of
 
 #ifdef DEBUG
 static void PrintStacks(PanexWidget w);
-static void 
+static void
 PrintTiles(PanexWidget w)
 #endif
 
@@ -109,6 +109,8 @@ PrintTiles(PanexWidget w)
 
 	static XtResource resourcesPanex[] =
 	{
+		{XtNuserName, XtCUserName, XtRString, sizeof (String),
+		 XtOffset(PanexWidget, panex.username), XtRString, "nobody"},
 		{XtNpyramidColor0, XtCLabel, XtRString, sizeof (String),
 	     XtOffset(PanexWidget, panex.pyramidName[0]), XtRString, "Blue"},
 		{XtNpyramidColor1, XtCLabel, XtRString, sizeof (String),
@@ -206,11 +208,19 @@ PrintTiles(PanexWidget w)
 		{2, 0}
 	};
 
-#ifndef HAS_USLEEP
+#ifndef HAVE_USLEEP
+#if !defined( VMS ) || defined( XVMSUTILS ) ||  ( __VMS_VER >= 70000000 )
+#ifdef USE_XVMSUTILS
+#include <X11/unix_time.h>
+#endif
+#if HAVE_SYS_TIME_H
 #include <sys/time.h>
-#ifdef AIXV3
+#else
+#if HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
+#endif
+#endif
 #if defined(SYSV) || defined(SVR4)
 #ifdef LESS_THAN_AIX3_2
 #include <sys/poll.h>
@@ -219,11 +229,11 @@ PrintTiles(PanexWidget w)
 #endif /* !LESS_THAN_AIX3_2 */
 #endif /* defined(SYSV) || defined(SVR4) */
 
-	static int
-	            usleep(long unsigned int usec)
+static int
+usleep(long unsigned int usec)
 {
 #if (defined (SYSV) || defined(SVR4)) && !defined(__hpux)
-#if defined(HAS_NANOSLEEP)
+#if defined(HAVE_NANOSLEEP)
 	{
 		struct timespec rqt;
 
@@ -248,14 +258,14 @@ PrintTiles(PanexWidget w)
 #else
 	struct timeval time_out;
 
-#if (!defined(AIXV3) && !defined(__hpux))
+#if  0				/* (!defined(AIXV3) && !defined(__hpux)) */
 	extern int  select(int, fd_set *, fd_set *, fd_set *, struct timeval *);
 
 #endif
 
 	time_out.tv_usec = usec % (unsigned long) 1000000;
 	time_out.tv_sec = usec / (unsigned long) 1000000;
-	(void) select(0, (fd_set *) 0, (fd_set *) 0, (fd_set *) 0, &time_out);
+	(void) select(0, (void *) 0, (void *) 0, (void *) 0, &time_out);
 #endif
 #endif
 	return 0;
@@ -537,12 +547,12 @@ GetPanex(PanexWidget w, XEvent * event, char **args, int nArgs)
 		SetStartPosition(w);
 		ScanMoves(fp, w, moves);
 		(void) fclose(fp);
-    if (mode == 0)
-      (void) printf("%s: mode hanoi, tiles %d, moves %d.\n", 
-        DATAFILE, tiles, moves);
-    else
-      (void) printf("%s: mode panex, tiles %d, moves %d.\n", 
-        DATAFILE, tiles, moves);
+		if (mode == 0)
+			(void) printf("%s: mode hanoi, tiles %d, moves %d.\n",
+				      DATAFILE, tiles, moves);
+		else
+			(void) printf("%s: mode panex, tiles %d, moves %d.\n",
+				      DATAFILE, tiles, moves);
 	}
 }
 
@@ -1000,16 +1010,16 @@ DrawPyramid(PanexWidget w, int color, int i, int j, int size, int offset)
 		char        buf[2];
 
 		XFillPolygon(XtDisplay(w), XtWindow(w), (offset) ? w->panex.tileGC :
-	       w->panex.inverseGC,
-		     trapazoidList, 4, Convex, CoordModePrevious);
+			     w->panex.inverseGC,
+			     trapazoidList, 4, Convex, CoordModePrevious);
 		(void) sprintf(buf, "%c", w->panex.pyramidName[color][0]);
 		XDrawString(XtDisplay(w), XtWindow(w),
-	        (offset) ? w->panex.inverseGC : w->panex.tileGC,
-	        k - w->panex.letterOffset.x, trapazoidList[0].y +
-          w->panex.tileSize.y / 2 + w->panex.letterOffset.y, buf, 1);
+			    (offset) ? w->panex.inverseGC : w->panex.tileGC,
+			    k - w->panex.letterOffset.x, trapazoidList[0].y +
+		  w->panex.tileSize.y / 2 + w->panex.letterOffset.y, buf, 1);
 	} else
 		XFillPolygon(XtDisplay(w), XtWindow(w), w->panex.pyramidGC[color],
-		     trapazoidList, 4, Convex, CoordModePrevious);
+			     trapazoidList, 4, Convex, CoordModePrevious);
 }
 
 Boolean
