@@ -92,7 +92,7 @@ extern long LongRNG(void);
 #define MAX(a,b) (((int)(a)>(int)(b))?(int)(a):(int)(b))
 
 typedef struct _RubikLoc {
-	int         face, rotation;
+	char         face, rotation;
 } RubikLoc;
 
 typedef struct _RubikPart {
