@@ -6,7 +6,7 @@
 #
 ###
 #
-#  Copyright (c) 1993 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
+#  Copyright (c) 1993 - 97	David Albert Bagley, bagleyd@bigfoot.com
 #
 #                   All Rights Reserved
 #
@@ -34,14 +34,20 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
 #ifdef VMS
 #include <unixlib.h>
-#define getlogin cuserid
+#define getlogin() cuserid(NULL)
 #else
-#ifndef apollo
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #endif
+#if HAVE_FCNTL_H
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#endif
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
@@ -69,6 +75,15 @@
 
 #define MAXCUBES 6
 #define MAXRECORD 32767
+#define FILENAMELEN 1024
+#define USERNAMELEN 128
+#define NOACCESS "noaccess"
+#define NOBODY "nobody"
+
+typedef struct {
+	int         score;
+	char        name[USERNAMELEN];
+} GameRecord;
 
 static void Initialize(void);
 static void CallbackRubik(Widget w, caddr_t clientData,
@@ -76,6 +91,7 @@ static void CallbackRubik(Widget w, caddr_t clientData,
 
 static void PrintRecord(int size, Boolean orient, Boolean practice);
 static int  HandleSolved(int counter, int size, Boolean orient);
+static void InitRecords(void);
 static void ReadRecords(void);
 static void WriteRecords(void);
 
@@ -90,8 +106,10 @@ static void motif_print(Widget w, char *text);
 static Arg  arg[4];
 static Widget moves, record, message, rubik2d, rubik3d, cubes, orientSwitch,
             practiceSwitch;
-static int  rubikRecord[2][MAXCUBES - MINCUBES + 1], movesDsp = 0;
+static GameRecord rubikRecord[2][MAXCUBES - MINCUBES + 1];
+static int  movesDsp = 0;
 static char messageDsp[128] = "Welcome";
+static char usernameDsp[USERNAMELEN] = "";
 static char buff[256];
 
 static void
@@ -101,34 +119,39 @@ usage(void)
 	(void) fprintf(stderr,
 	     "\t[-geometry [{width}][x{height}][{+-}{xoff}[{+-}{yoff}]]]\n");
 	(void) fprintf(stderr,
-		"\t[-display [{host}]:[{vs}]][-fg {color}] [-bg {color}]\n");
+		       "\t[-display [{host}]:[{vs}]] [-mono]\n");
 	(void) fprintf(stderr,
-	   "\t[-{border|bd} {color}] [-mono] [-size {int}] [-[no]orient]\n");
+		"\t[-{foreground|fg} {color}] [-{background|bg} {color}]\n");
 	(void) fprintf(stderr,
-		       "\t[-[no]practice] [-face{0|1|2|3|4|5} {color}]\n");
+		  "\t[-{border|bd} {color}] [-face{0|1|2|3|4|5} {color}]\n");
+	(void) fprintf(stderr,
+		       "\t[-size {int}] [-[no]orient] [-[no]practice]\n");
+	(void) fprintf(stderr,
+		       "\t[-username {string}]\n");
 	exit(1);
 }
 
 static XrmOptionDescRec options[] =
 {
-	{"-fg", "rubik.Foreground", XrmoptionSepArg, NULL},
-	{"-bg", "*Background", XrmoptionSepArg, NULL},
-	{"-foreground", "rubik.Foreground", XrmoptionSepArg, NULL},
-	{"-background", "*Background", XrmoptionSepArg, NULL},
-	{"-border", "*rubik.pieceBorder", XrmoptionSepArg, NULL},
-	{"-bd", "*rubik.pieceBorder", XrmoptionSepArg, NULL},
 	{"-mono", "*rubik.mono", XrmoptionNoArg, "TRUE"},
-	{"-size", "*rubik.size", XrmoptionSepArg, NULL},
-	{"-orient", "*rubik.orient", XrmoptionNoArg, "TRUE"},
-	{"-noorient", "*rubik.orient", XrmoptionNoArg, "FALSE"},
-	{"-practice", "*rubik.practice", XrmoptionNoArg, "TRUE"},
-	{"-nopractice", "*rubik.practice", XrmoptionNoArg, "FALSE"},
+	{"-fg", "rubik.Foreground", XrmoptionSepArg, NULL},
+	{"-foreground", "rubik.Foreground", XrmoptionSepArg, NULL},
+	{"-bg", "*Background", XrmoptionSepArg, NULL},
+	{"-background", "*Background", XrmoptionSepArg, NULL},
+	{"-bd", "*rubik.pieceBorder", XrmoptionSepArg, NULL},
+	{"-border", "*rubik.pieceBorder", XrmoptionSepArg, NULL},
 	{"-face0", "*rubik.faceColor0", XrmoptionSepArg, NULL},
 	{"-face1", "*rubik.faceColor1", XrmoptionSepArg, NULL},
 	{"-face2", "*rubik.faceColor2", XrmoptionSepArg, NULL},
 	{"-face3", "*rubik.faceColor3", XrmoptionSepArg, NULL},
 	{"-face4", "*rubik.faceColor4", XrmoptionSepArg, NULL},
 	{"-face5", "*rubik.faceColor5", XrmoptionSepArg, NULL},
+	{"-size", "*rubik.size", XrmoptionSepArg, NULL},
+	{"-orient", "*rubik.orient", XrmoptionNoArg, "TRUE"},
+	{"-noorient", "*rubik.orient", XrmoptionNoArg, "FALSE"},
+	{"-practice", "*rubik.practice", XrmoptionNoArg, "TRUE"},
+	{"-nopractice", "*rubik.practice", XrmoptionNoArg, "FALSE"},
+	{"-username", "*rubik.userName", XrmoptionSepArg, NULL}
 };
 
 int
@@ -256,13 +279,14 @@ main(int argc, char **argv)
 /* There's probably a better way to assure that they are the same but I don't
    know it off hand. */
 static void
-MakeEquivalent(int *size, Boolean * orient, Boolean * practice)
+MakeEquivalent(String * username, int *size, Boolean * orient, Boolean * practice)
 {
 	Boolean     mono;
 	Pixel       foreground, background, pieceBorder;
 	String      faceColor[MAXFACES];
 
 	XtVaGetValues(rubik2d,
+		      XtNuserName, username,
 		      XtNsize, size,
 		      XtNorient, orient,
 		      XtNpractice, practice,
@@ -282,6 +306,7 @@ MakeEquivalent(int *size, Boolean * orient, Boolean * practice)
 		      XtNstart, FALSE,
 		      NULL);
 	XtVaSetValues(rubik3d,
+		      XtNuserName, *username,
 		      XtNsize, *size,
 		      XtNorient, *orient,
 		      XtNmono, mono,
@@ -305,13 +330,23 @@ Initialize(void)
 {
 	int         size;
 	Boolean     orient, practice;
+	String      username;
 
-	MakeEquivalent(&size, &orient, &practice);
+	MakeEquivalent(&username, &size, &orient, &practice);
 	if (size <= MAXCUBES)
 		XmScaleSetValue(cubes, size);
 	XmToggleButtonSetState(orientSwitch, orient, True);
 	XmToggleButtonSetState(practiceSwitch, practice, True);
+	InitRecords();
 	ReadRecords();
+	(void) strcpy(usernameDsp, username);
+	if (!strcmp(usernameDsp, "") || !strcmp(usernameDsp, NOACCESS) ||
+	    !strcmp(usernameDsp, NOBODY)) {
+		/* The NOACCESS is not necasary, but it stops people from being cute. */
+		(void) sprintf(usernameDsp, "%s", getlogin());
+		if (!strcmp(usernameDsp, "") || !strcmp(usernameDsp, NOACCESS))
+			(void) sprintf(usernameDsp, "%s", NOBODY);	/* It really IS nobody */
+	}
 	PrintRecord(size, orient, practice);
 }
 
@@ -369,7 +404,7 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 				movesDsp = 0;
 			else {
 				if (HandleSolved(movesDsp, size, orient))
-					(void) sprintf(messageDsp, "Congratulations %s!!", getlogin());
+					(void) sprintf(messageDsp, "Congratulations %s!!", usernameDsp);
 				else
 					(void) strcpy(messageDsp, "Solved!");
 			}
@@ -528,10 +563,12 @@ PrintRecord(int size, Boolean orient, Boolean practice)
 		motif_print(record, "practice");
 	else if (size > MAXCUBES)
 		motif_print(record, "NOT RECORDED");
-	else if (rubikRecord[i][j] >= MAXRECORD)
-		motif_print(record, "NEVER");
-	else {
-		(void) sprintf(buff, "%d", rubikRecord[i][j]);
+	else if (rubikRecord[i][j].score >= MAXRECORD) {
+		(void) sprintf(buff, "NEVER %s", NOACCESS);
+		motif_print(record, buff);
+	} else {
+		(void) sprintf(buff, "%d %s",
+			    rubikRecord[i][j].score, rubikRecord[i][j].name);
 		motif_print(record, buff);
 	}
 }
@@ -542,10 +579,13 @@ HandleSolved(int counter, int size, Boolean orient)
 	int         i = (orient) ? 1 : 0;
 	int         j = size - MINCUBES;
 
-	if (size <= MAXCUBES && counter < rubikRecord[i][j]) {
-		rubikRecord[i][j] = counter;
-		if (size < 2 || (orient && rubikRecord[i][j] < rubikRecord[!i][j]))
-			rubikRecord[!i][j] = counter;
+	if (size <= MAXCUBES && counter < rubikRecord[i][j].score) {
+		rubikRecord[i][j].score = counter;
+		(void) strcpy(rubikRecord[i][j].name, usernameDsp);
+		if (size < 2 || (orient && counter < rubikRecord[!i][j].score)) {
+			rubikRecord[!i][j].score = counter;
+			(void) strcpy(rubikRecord[!i][j].name, usernameDsp);
+		}
 		WriteRecords();
 		PrintRecord(size, orient, FALSE);
 		return TRUE;
@@ -553,23 +593,36 @@ HandleSolved(int counter, int size, Boolean orient)
 	return FALSE;
 }
 
+static void
+InitRecords(void)
+{
+	int         i, orient;
+
+	for (orient = 0; orient < 2; orient++)
+		for (i = 0; i < MAXCUBES - MINCUBES + 1; i++) {
+			rubikRecord[orient][i].score = MAXRECORD;
+			(void) strcpy(rubikRecord[orient][i].name, NOACCESS);
+		}
+}
+
 static void
 ReadRecords(void)
 {
 	FILE       *fp;
 	int         i, n, orient;
+	char        username[USERNAMELEN];
 
-	for (orient = 0; orient < 2; orient++)
-		for (i = 0; i < MAXCUBES - MINCUBES + 1; i++)
-			rubikRecord[orient][i] = MAXRECORD;
 	if ((fp = fopen(SCOREFILE, "r")) == NULL) {
 		(void) sprintf(buff, "Can not open %s, taking defaults.", SCOREFILE);
 		motif_print(message, buff);
 	} else {
 		for (orient = 0; orient < 2; orient++)
 			for (i = 0; i < MAXCUBES - MINCUBES + 1; i++) {
-				(void) fscanf(fp, "%d", &n);
-				rubikRecord[orient][i] = n;
+				(void) fscanf(fp, "%d %s\n", &n, username);
+				if (n <= rubikRecord[orient][i].score) {
+					rubikRecord[orient][i].score = n;
+					(void) strcpy(rubikRecord[orient][i].name, username);
+				}
 			}
 		(void) fclose(fp);
 	}
@@ -585,11 +638,35 @@ WriteRecords(void)
 		(void) sprintf(buff, "Can not write to %s.", SCOREFILE);
 		motif_print(message, buff);
 	} else {
+#if HAVE_FCNTL_H
+		int         lfd;
+		char        lockfile[FILENAMELEN];
+
+		(void) strcpy(lockfile, SCOREFILE);
+		(void) strcat(lockfile, ".lock");
+		while (((lfd = open(lockfile, O_CREAT | O_EXCL, 0644)) < 0) &&
+		       errno == EEXIST)
+			(void) sleep(1);
+		if (lfd < 0) {
+#if 1
+			(void) fprintf(stderr, "Lock file exists... guessing its an old one.\n");
+#else
+			(void) fprintf(stderr, "Lock file exists... score not recorded - sorry.\n"
+				);
+			return;
+#endif
+		}
+#endif
 		for (orient = 0; orient < 2; orient++) {
 			for (i = 0; i < MAXCUBES - MINCUBES + 1; i++)
-				(void) fprintf(fp, "%d ", rubikRecord[orient][i]);
+				(void) fprintf(fp, "%d %s\n",
+					       rubikRecord[orient][i].score, rubikRecord[orient][i].name);
 			(void) fprintf(fp, "\n");
 		}
+#if HAVE_FCNTL_H
+		(void) close(lfd);
+		(void) unlink(lockfile);
+#endif
 		(void) fclose(fp);
 	}
 }
