@@ -6,7 +6,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
+#  Copyright (c) 1994 - 97	David Albert Bagley, bagleyd@bigfoot.com
 #
 #                   All Rights Reserved
 #
@@ -36,6 +36,7 @@
  ***********************************************************************/
 
 #define XtNselectCallback "selectCallback"
+#define XtNuserName "userName"
 #define XtNorient "orient"
 #define XtNmono "mono"
 #define XtNface "face"
@@ -50,6 +51,7 @@
 #define XtNfaceColor3 "faceColor3"
 #define XtNfaceColor4 "faceColor4"
 #define XtNfaceColor5 "faceColor5"
+#define XtCUserName "UserName"
 #define XtCOrient "Orient"
 #define XtCMono "Mono"
 #define XtCFace "Face"
