@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
+#  Copyright (c) 1994 - 97	David Albert Bagley, bagleyd@bigfoot.com
 #
 #                   All Rights Reserved
 #
@@ -30,7 +30,7 @@
 #ifdef VMS
 #include <unixlib.h>
 #else
-#ifndef apollo
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #endif
@@ -140,6 +140,8 @@ static XtActionsRec actionsListTriangles[] =
 
 static XtResource resourcesTriangles[] =
 {
+	{XtNuserName, XtCUserName, XtRString, sizeof (String),
+	 XtOffset(TrianglesWidget, triangles.username), XtRString, "nobody"},
 	{XtNforeground, XtCForeground, XtRPixel, sizeof (Pixel),
 	 XtOffset(TrianglesWidget, triangles.foreground), XtRString,
 	 XtDefaultForeground},
